char* headmsg = "\
##########################################\n\
# MPEGTool statistics gnuplot macro file #\n\
##########################################\n";

char* general_str = "\
set size 1.0,1.0\n\
set noyzeroaxis\n";

char* label_str = "\
set title  \"%s\"\n\
set xlabel \"%s\"\n\
set ylabel \"%s\"\n";

char *scope_str[] = {
"Frame",
"Slice",
"Bits"
};

char*atm_mode_str[] = {
"Bits",
"ATM Packets",
};

char* xrange_str = "set xrange [%f:%f]\n";
char* yrange_str = "set yrange [%f:%f]\n";

char* plot_str[] = {
"plot   \"%s\" using 1:2  title \"HP\" with lines 1,\
 \"%s\" using 1:3  title \"LP\" with lines 2\n",
"plot   \"%s\" using 1:2 title \"HP\" with lines 1\n",
"plot   \"%s\" using 1:3 title \"LP\" with lines 1\n",
"plot   \"%s\" using 1:4 title \"HP+LP\" with lines 1\n"
};

char* dist_plot_str[] = {
"plot   \"%s\" using 1:2:3:4  title \"HP\" with errorbars,\
 \"%s\" using 1:5:6:7  title \"LP\" with errorbars\n",
"plot   \"%s\" using 1:2:3:4 title \"HP\" with errorbars\n",
"plot   \"%s\" using 1:5:6:7 title \"LP\" with errorbars\n",
"plot   \"%s\" using 1:8:9:10 title \"HP+LP\" with errorbars\n",
};

char* pause_str = "\
pause -1 \"GNUPLOT: Hit any key\"\n";

char* stat_str[] = {
"HP : Mean = %10.2f, Std. Dev. = %10.2f, Max. = %7d",
"HP : Mean = %7.2f, Std. Dev. = %7.2f, Max. = %5d",
"LP : Mean = %10.2f, Std. Dev. = %10.2f, Max. = %7d",
"LP : Mean = %7.2f, Std. Dev. = %7.2f, Max. = %5d",
"Mean = %10.2f, Std. Dev. = %10.2f, Max. = %7d",
"Mean = %7.2f, Std. Dev. = %7.2f, Max. = %5d"
};

char* ps_str = "\
set terminal postscript default 12\n\
set output \"%s\"\n\
replot\n";

/* for small PS output */
char* ps_str2 = "\
set terminal postscript eps 10\n\
set output \"%s\"\n\
replot\n";
