#define MAX_ERROR_MESSAGE 80
char msg[MAX_ERROR_MESSAGE+1][256] = {
/*  0 */ " Undefined error",
/*  1 */ "%s", /* user created error message */
/*------ File related errors ------*/
/*  2 */ " File cannot be opened (%s)",
/*  3 */ " File not opened",
/*  4 */ " File read error",
/*  5 */ " Invalid selection",
/*  6 */ " Block size in tape doesn't match",
/*  7 */ " Data read error",
/*  8 */ " ",
/*  9 */ " ",
/* 10 */ " ",
/*------ encoder related ------*/
/* 11 */ " N must be M multiply by integer.",
/* 12 */ " Range of Beta is between 1 and 64.",
/* 13 */ " Width/height mustbe positive",
/* 14 */ " Number of Frames must be [M * int + 1].",
/* 15 */ " Quantaization factor should be 1 - 32",
/* 16 */ " M must be smaller than or equal to N",
/* 17 */ " Invalid frame number",
/* 18 */ " N or M must be postitve",
/* 19 */ " Please specify data name",
/* 20 */ " ",
/* 21 */ " Please specify input video data file",
/* 22 */ " Please specify input video data device",
/* 23 */ " Please specify output video data device",
/* 24 */ " Can't open video data file/device (%s)",
/* 25 */ " Can't open mpeg data output file/device (%s)",
/* 26 */ " File skip error, too many retries",
/* 27 */ " Not enough video data. EOF has been reached while skipping",
/* 28 */ " ",
/* 29 */ " Not enough video data. EOF has been reached while reading data",
/* 30 */ " Encoder Terminated !!!",
/* 31 */ " ",
/* 32 */ " ",
/* 33 */ " ",
/* 34 */ " ",
/* 35 */ " ",
/* 36 */ " ",
/* 37 */ " ",
/* 38 */ " ",
/* 39 */ " ",
/* 40 */ " ",
/* 41 */ " ",
/* 42 */ " ",
/* 43 */ " ",
/* 44 */ " ",
/* 45 */ " ",
/* 46 */ " ",
/* 47 */ " ",
/* 48 */ " ",
/* 49 */ " ",
/*------ stat related ------*/
/* 50 */ " Statistics Parameter Error",
/* 51 */ " Start frame number must be more than 0",
/* 52 */ " Number of statistics must be more than 0",
/* 53 */ " Please specify statistics file name",
/* 54 */ " Please specify postscript file name",
/* 55 */ " Improper MPEGTool statistics file",
/* 56 */ " Not enough number of data exists in the statistics file",
/* 57 */ " Statistics Data read error while reading",
/* 58 */ " Statistics File exists but file size is 0",
/* 59 */ " No Low Priority (LP) stream exist.",
/* 50 */ " ",
/* 61 */ " ",
/* 62 */ " ",
/* 63 */ " ",
/* 64 */ " ",
/* 65 */ " ",
/* 66 */ " ",
/* 67 */ " ",
/* 68 */ " ",
/* 69 */ " ",
/* 70 */ " ",
/* 71 */ " ",
/* 72 */ " ",
/* 73 */ " ",
/* 74 */ " ",
/* 75 */ " ",
/* 76 */ " ",
/* 77 */ " ",
/* 78 */ " ",
/* 79 */ " ",
/* 80 */ " ",
};

