#define MBSIZE 16

#ifndef DISPLACE
#define DISPLACE 15 /* define search area of motion vector */
#endif

typedef struct {
  int xdisp;
  int ydisp;
  int error;
} MV;

/* global info on picture */

int ncols,
    nlines;

MV    *get_mv();
int   get_abd();
byte  *get_mblock();
MV    *exhaustive();
MV    *fast();


